@@ -1,20 +1,5 @@
-#!/usr/bin/python
-# -*- coding: UTF-8 -*-
-"""
-
-References:
-    [1] Haoming Li et al. Matrix Completion via Alternating Least Square(ALS)
-        (https://stanford.edu/~rezab/classes/cme323/S15/notes/lec14.pdf)
-    [2] Yifan Hu et al. Collaborative Filtering for Implicit Feedback Datasets
-        (http://yifanhu.net/PUB/cf.pdf)
-    [3] Gábor Takács et al. Applications of the Conjugate Gradient Method for Implicit Feedback Collaborative Filtering
-        (http://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.379.6473&rep=rep1&type=pdf)
-
-author: massquantity
-
-"""
+"""Implementation of Alternating Least Squares."""
 import logging
-import numbers
 import os
 from functools import partial
 
@@ -26,45 +11,76 @@ from ..recommendation import recommend_from_embedding
 from ..utils.initializers import truncated_normal
 from ..utils.misc import time_block
 from ..utils.save_load import save_default_recs, save_params
-from ..utils.validate import check_has_sampled
 
-try:
-    from ._als import als_update
-except ImportError:
-    LOG_FORMAT = "%(asctime)s - %(levelname)s - %(message)s"
-    logging.basicConfig(format=LOG_FORMAT)
-    logging.warning("Als cython version is not available")
-    raise
+LOG_FORMAT = "%(asctime)s - %(levelname)s - %(message)s"
+logging.basicConfig(format=LOG_FORMAT)
 
 
 class ALS(EmbedBase):
+    """*Alternating Least Squares* algorithm.
+
+    One can use conjugate gradient optimization and set more `n_threads`
+    to accelerate training.
+
+    Parameters
+    ----------
+    task : {'rating', 'ranking'}
+        Recommendation task. See :ref:`Task`.
+    data_info : :class:`~libreco.data.DataInfo` object
+          Object that contains useful information for training and inference.
+    embed_size : int, default: 16
+        Vector size of embeddings.
+    n_epochs : int, default: 10
+        Number of epochs for training.
+    reg : float or None, default: None
+        Regularization parameter, must be non-negative or None.
+    alpha : int, default: 10
+        Parameter used for increasing confidence level, only applied for `ranking` task.
+    use_cg : bool, default: True
+        Whether to use *conjugate gradient* optimization. See `reference <http://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.379.6473&rep=rep1&type=pdf>`_.
+    n_threads : int, default: 1
+        Number of threads to use.
+    seed : int, default: 42
+        Random seed.
+    lower_upper_bound : tuple or None, default: None
+        Lower and upper score bound for `rating` task.
+
+    References
+    ----------
+    [1] *Haoming Li et al.* `Matrix Completion via Alternating Least Square(ALS)
+    <https://stanford.edu/~rezab/classes/cme323/S15/notes/lec14.pdf>`_.
+
+    [2] *Yifan Hu et al.* `Collaborative Filtering for Implicit Feedback Datasets
+    <http://yifanhu.net/PUB/cf.pdf>`_.
+
+    [3] *Gábor Takács et al.* `Applications of the Conjugate Gradient Method for Implicit Feedback Collaborative Filtering
+    <http://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.379.6473&rep=rep1&type=pdf>`_.
+    """
+
     def __init__(
         self,
         task,
-        data_info=None,
+        data_info,
         embed_size=16,
-        n_epochs=20,
+        n_epochs=10,
         reg=None,
         alpha=10,
         use_cg=True,
         n_threads=1,
         seed=42,
         lower_upper_bound=None,
-        with_training=True,
     ):
         super().__init__(task, data_info, embed_size, lower_upper_bound)
 
         self.all_args = locals()
         self.n_epochs = n_epochs
-        self.reg = reg
+        self.reg = self._check_reg(reg)
         self.alpha = alpha
         self.use_cg = use_cg
         self.n_threads = n_threads
         self.seed = seed
-        if with_training:
-            self._build_model()
 
-    def _build_model(self):
+    def build_model(self):
         np.random.seed(self.seed)
         self.user_embed = truncated_normal(
             shape=[self.n_users, self.embed_size], mean=0.0, scale=0.03
@@ -80,17 +96,46 @@ class ALS(EmbedBase):
         shuffle=True,
         eval_data=None,
         metrics=None,
-        **kwargs,
+        k=10,
+        eval_batch_size=8192,
+        eval_user_num=None,
     ):
-        k = kwargs.get("k", 10)
-        eval_batch_size = kwargs.get("eval_batch_size", 2**15)
-        eval_user_num = kwargs.get("eval_user_num", None)
+        """Fit ALS model on the training data.
+
+        Parameters
+        ----------
+        train_data : :class:`~libreco.data.TransformedSet` object
+            Data object used for training.
+        verbose : int, default: 1
+            Print verbosity. If `eval_data` is provided, setting it to higher than 1
+            will print evaluation metrics during training.
+        shuffle : bool, default: True
+            Whether to shuffle the training data.
+        eval_data : :class:`~libreco.data.TransformedSet` object, default: None
+            Data object used for evaluating.
+        metrics : list or None, default: None
+            List of metrics for evaluating.
+        k : int, default: 10
+            Parameter of metrics, e.g. recall at k, ndcg at k
+        eval_batch_size : int, default: 8192
+            Batch size for evaluating.
+        eval_user_num : int or None, default: None
+            Number of users for evaluating. Setting it to a positive number will sample
+            users randomly from eval data.
+        """
+        try:
+            from ._als import als_update
+        except (ImportError, ModuleNotFoundError):
+            logging.warning("Als cython version is not available")
+            raise
+
+        self.check_attribute(eval_data, k)
         self.show_start_time()
-        assert isinstance(self.reg, numbers.Real), "`reg` must be float"
+        self.build_model()
         user_interaction = train_data.sparse_interaction  # sparse.csr_matrix
         item_interaction = user_interaction.T.tocsr()
         if self.task == "ranking":
-            check_has_sampled(train_data, verbose)
+            # check_has_sampled(train_data, verbose)
             user_interaction.data = user_interaction.data * self.alpha + 1
             item_interaction.data = item_interaction.data * self.alpha + 1
 
@@ -124,7 +169,6 @@ class ALS(EmbedBase):
                     seed=self.seed,
                 )
                 print("=" * 30)
-        self.set_embeddings()
         self.assign_embedding_oov()
         self.default_recs = recommend_from_embedding(
             task=self.task,
@@ -137,7 +181,22 @@ class ALS(EmbedBase):
             random_rec=False,
         ).flatten()
 
+    @staticmethod
+    def _check_reg(reg):
+        if not isinstance(reg, float) or reg <= 0.0:
+            raise ValueError(f"`reg` must be float and positive, got {reg}")
+        return reg
+
     def save(self, path, model_name, **kwargs):
+        """Save model for inference or retraining.
+
+        Parameters
+        ----------
+        path : str
+            File folder path to save model.
+        model_name : str
+            Name of the saved model file.
+        """
         if not os.path.isdir(path):
             print(f"file folder {path} doesn't exists, creating a new one...")
             os.makedirs(path)
@@ -148,10 +207,19 @@ class ALS(EmbedBase):
             variable_path, user_embed=self.user_embed, item_embed=self.item_embed
         )
 
-    def set_embeddings(self):
+    def set_embeddings(self):  # pragma: no cover
         pass
 
     def rebuild_model(self, path, model_name):
+        """Reconstruct model for retraining.
+
+        Parameters
+        ----------
+        path : str
+            File folder path for saved model.
+        model_name : str
+            Name of the saved model file.
+        """
         variable_path = os.path.join(path, f"{model_name}.npz")
         variables = np.load(variable_path)
         # remove oov values
@@ -162,6 +230,7 @@ class ALS(EmbedBase):
 
 
 def least_squares(sparse_interaction, X, Y, reg, embed_size, num, mode):
+    """Least squares optimization showcase for ALS."""
     indices = sparse_interaction.indices
     indptr = sparse_interaction.indptr
     data = sparse_interaction.data
@@ -191,6 +260,7 @@ def least_squares(sparse_interaction, X, Y, reg, embed_size, num, mode):
 
 # O(f^3) * m
 def least_squares_cg(sparse_interaction, X, Y, reg, embed_size, num, mode, cg_steps=3):
+    """Conjugate Gradient optimization showcase for ALS."""
     indices = sparse_interaction.indices
     indptr = sparse_interaction.indptr
     data = sparse_interaction.data
