@@ -1,40 +1,77 @@
-"""
-
-References: Steffen Rendle et al. "BPR: Bayesian Personalized Ranking from Implicit Feedback"
-            (https://arxiv.org/ftp/arxiv/papers/1205/1205.2618.pdf)
-
-author: massquantity
-
-"""
+"""Implementation of Bayesian Personalized Ranking."""
 import logging
 from functools import partial
 
 import numpy as np
-from tensorflow.keras.initializers import (
-    zeros as tf_zeros,
-    truncated_normal as tf_truncated_normal,
-)
 
 from ..bases import EmbedBase, ModelMeta
 from ..evaluation import print_metrics
 from ..recommendation import recommend_from_embedding
 from ..tfops import reg_config, sess_config, tf
-from ..training import BPRTrainer
+from ..training import get_trainer
 from ..utils.initializers import truncated_normal
 from ..utils.misc import time_block
-from ..utils.validate import check_has_sampled
 
-try:
-    from ._bpr import bpr_update
-except (ImportError, ModuleNotFoundError):
-    LOG_FORMAT = "%(asctime)s - %(levelname)s - %(message)s"
-    logging.basicConfig(format=LOG_FORMAT)
-    logging.warning("BPR cython version is not available")
+LOG_FORMAT = "%(asctime)s - %(levelname)s - %(message)s"
+logging.basicConfig(format=LOG_FORMAT)
 
 
 class BPR(EmbedBase, metaclass=ModelMeta, backend="tensorflow"):
-    """
-    BPR is only suitable for ranking task
+    """*Bayesian Personalized Ranking* algorithm.
+
+    *BPR* is implemented in both TensorFlow and Cython.
+
+    .. CAUTION::
+        + BPR can only be used in ``ranking`` task.
+        + BPR can only use ``bpr`` loss in ``loss_type``.
+
+    Parameters
+    ----------
+    task : {'ranking'}
+        Recommendation task. See :ref:`Task`.
+    data_info : :class:`~libreco.data.DataInfo` object
+        Object that contains useful information for training and inference.
+    loss_type : {'bpr'}
+        Loss for model training.
+    embed_size: int, default: 16
+        Vector size of embeddings.
+    n_epochs: int, default: 10
+        Number of epochs for training.
+    lr : float, default 0.001
+        Learning rate for training.
+    lr_decay : bool, default: False
+        Whether to use learning rate decay.
+    epsilon : float, default: 1e-5
+        A small constant added to the denominator to improve numerical stability in
+        Adam optimizer.
+        According to the `official comment <https://github.com/tensorflow/tensorflow/blob/v1.15.0/tensorflow/python/training/adam.py#L64>`_,
+        default value of `1e-8` for `epsilon` is generally not good, so here we choose `1e-5`.
+        Users can try tuning this hyperparameter if the training is unstable.
+    reg : float or None, default: None
+        Regularization parameter, must be non-negative or None.
+    batch_size : int, default: 256
+        Batch size for training.
+    num_neg : int, default: 1
+        Number of negative samples for each positive sample, only used in `ranking` task.
+    use_tf : bool, default: True
+        Whether to use TensorFlow or Cython version. The TensorFlow version is more
+        accurate, whereas the Cython version is faster.
+    seed : int, default: 42
+        Random seed.
+    lower_upper_bound : tuple or None, default: None
+        Lower and upper score bound for `rating` task.
+    tf_sess_config : dict or None, default: None
+        Optional TensorFlow session config, see `ConfigProto options
+        <https://github.com/tensorflow/tensorflow/blob/v2.10.0/tensorflow/core/protobuf/config.proto#L431>`_.
+    optimizer : {'sgd', 'momentum', 'adam'}, default: 'adam'
+        Optimizer used in Cython version.
+    num_threads : int, default: 1
+        Number of threads used in Cython version.
+
+    References
+    ----------
+    *Steffen Rendle et al.* `BPR: Bayesian Personalized Ranking from Implicit Feedback
+    <https://arxiv.org/ftp/arxiv/papers/1205/1205.2618.pdf>`_.
     """
 
     user_variables = ["user_embed_var"]
@@ -59,37 +96,28 @@ class BPR(EmbedBase, metaclass=ModelMeta, backend="tensorflow"):
         tf_sess_config=None,
         optimizer="adam",
         num_threads=1,
-        with_training=True,
     ):
         super().__init__(task, data_info, embed_size)
 
         assert task == "ranking", "BPR is only suitable for ranking"
         assert loss_type == "bpr", "BPR should use bpr loss"
         self.all_args = locals()
-        self.reg = reg_config(reg) if use_tf else reg
+        self.loss_type = loss_type
         self.n_epochs = n_epochs
         self.lr = lr
+        self.lr_decay = lr_decay
+        self.epsilon = epsilon
+        self.reg = reg_config(reg) if use_tf else reg
+        self.batch_size = batch_size
+        self.num_neg = num_neg
         self.use_tf = use_tf
         self.seed = seed
         self.optimizer = optimizer
         self.num_threads = num_threads
-        if with_training:
-            self._build_model()
-            if use_tf:
-                self.sess = sess_config(tf_sess_config)
-                self.tf_trainer = BPRTrainer(
-                    self,
-                    task,
-                    loss_type,
-                    n_epochs,
-                    lr,
-                    lr_decay,
-                    epsilon,
-                    batch_size,
-                    num_neg,
-                )
+        if use_tf:
+            self.sess = sess_config(tf_sess_config)
 
-    def _build_model(self):
+    def build_model(self):
         if self.use_tf:
             self._build_model_tf()
         else:
@@ -115,19 +143,19 @@ class BPR(EmbedBase, metaclass=ModelMeta, backend="tensorflow"):
         self.user_embed_var = tf.get_variable(
             name="user_embed_var",
             shape=[self.n_users, self.embed_size],
-            initializer=tf_truncated_normal(0.0, 0.03),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
         self.item_embed_var = tf.get_variable(
             name="item_embed_var",
             shape=[self.n_items, self.embed_size],
-            initializer=tf_truncated_normal(0.0, 0.03),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
         self.item_bias_var = tf.get_variable(
             name="item_bias_var",
             shape=[self.n_items],
-            initializer=tf_zeros,
+            initializer=tf.zeros_initializer(),
             regularizer=self.reg,
         )
 
@@ -157,15 +185,43 @@ class BPR(EmbedBase, metaclass=ModelMeta, backend="tensorflow"):
         shuffle=True,
         eval_data=None,
         metrics=None,
-        **kwargs,
+        k=10,
+        eval_batch_size=8192,
+        eval_user_num=None,
     ):
-        k = kwargs.get("k", 10)
-        eval_batch_size = kwargs.get("eval_batch_size", 2**15)
-        eval_user_num = kwargs.get("eval_user_num", None)
+        """Fit BPR model on the training data.
+
+        Parameters
+        ----------
+        train_data : :class:`~libreco.data.TransformedSet` object
+            Data object used for training.
+        verbose : int, default: 1
+            Print verbosity. If `eval_data` is provided, setting it to higher than 1
+            will print evaluation metrics during training.
+        shuffle : bool, default: True
+            Whether to shuffle the training data.
+        eval_data : :class:`~libreco.data.TransformedSet` object, default: None
+            Data object used for evaluating.
+        metrics : list or None, default: None
+            List of metrics for evaluating.
+        k : int, default: 10
+            Parameter of metrics, e.g. recall at k, ndcg at k
+        eval_batch_size : int, default: 8192
+            Batch size for evaluating.
+        eval_user_num : int or None, default: None
+            Number of users for evaluating. Setting it to a positive number will sample
+            users randomly from eval data.
+        """
+        self.check_attribute(eval_data, k)
         self.show_start_time()
-        check_has_sampled(train_data, verbose)
+        # check_has_sampled(train_data, verbose)
+        if not self.model_built:
+            self.build_model()
+            self.model_built = True
         if self.use_tf:
-            self.tf_trainer.run(
+            if self.trainer is None:
+                self.trainer = get_trainer(self)
+            self.trainer.run(
                 train_data,
                 verbose,
                 shuffle,
@@ -210,6 +266,12 @@ class BPR(EmbedBase, metaclass=ModelMeta, backend="tensorflow"):
         eval_batch_size=None,
         eval_user_num=None,
     ):
+        try:
+            from ._bpr import bpr_update
+        except (ImportError, ModuleNotFoundError):
+            logging.warning("BPR cython version is not available")
+            raise
+
         if self.optimizer == "sgd":
             trainer = partial(bpr_update)
 
