@@ -1,13 +1,4 @@
-"""
-
-Reference: Huifeng Guo et al. "DeepFM: A Factorization-Machine based Neural Network for CTR Prediction"
-           (https://arxiv.org/pdf/1703.04247.pdf)
-
-author: massquantity
-
-"""
-from tensorflow.keras.initializers import truncated_normal as tf_truncated_normal
-
+"""Implementation of DeepFM."""
 from ..bases import ModelMeta, TfBase
 from ..tfops import (
     dense_nn,
@@ -17,7 +8,7 @@ from ..tfops import (
     tf,
     tf_dense,
 )
-from ..training import TensorFlowTrainer
+from ..torchops import hidden_units_config
 from ..utils.misc import count_params
 from ..utils.validate import (
     check_dense_values,
@@ -31,6 +22,62 @@ from ..utils.validate import (
 
 
 class DeepFM(TfBase, metaclass=ModelMeta):
+    """*DeepFM* algorithm.
+
+    Parameters
+    ----------
+    task : {'rating', 'ranking'}
+        Recommendation task. See :ref:`Task`.
+    data_info : :class:`~libreco.data.DataInfo` object
+        Object that contains useful information for training and inference.
+    loss_type : {'cross_entropy', 'focal'}, default: 'cross_entropy'
+        Loss for model training.
+    embed_size: int, default: 16
+        Vector size of embeddings.
+    n_epochs: int, default: 10
+        Number of epochs for training.
+    lr : float, default 0.001
+        Learning rate for training.
+    lr_decay : bool, default: False
+        Whether to use learning rate decay.
+    epsilon : float, default: 1e-5
+        A small constant added to the denominator to improve numerical stability in
+        Adam optimizer.
+        According to the `official comment <https://github.com/tensorflow/tensorflow/blob/v1.15.0/tensorflow/python/training/adam.py#L64>`_,
+        default value of `1e-8` for `epsilon` is generally not good, so here we choose `1e-5`.
+        Users can try tuning this hyperparameter if the training is unstable.
+    reg : float or None, default: None
+        Regularization parameter, must be non-negative or None.
+    batch_size : int, default: 256
+        Batch size for training.
+    num_neg : int, default: 1
+        Number of negative samples for each positive sample, only used in `ranking` task.
+    use_bn : bool, default: True
+        Whether to use batch normalization.
+    dropout_rate : float or None, default: None
+        Probability of an element to be zeroed. If it is None, dropout is not used.
+    hidden_units : int, list of int or tuple of (int,), default: (128, 64, 32)
+        Number of layers and corresponding layer size in MLP.
+
+        .. versionchanged:: 1.0.0
+           Accept type of ``int``, ``list`` or ``tuple``, instead of ``str``.
+
+    multi_sparse_combiner : {'normal', 'mean', 'sum', 'sqrtn'}, default: 'sqrtn'
+        Options for combining `multi_sparse` features.
+    seed : int, default: 42
+        Random seed.
+    lower_upper_bound : tuple or None, default: None
+        Lower and upper score bound for `rating` task.
+    tf_sess_config : dict or None, default: None
+        Optional TensorFlow session config, see `ConfigProto options
+        <https://github.com/tensorflow/tensorflow/blob/v2.10.0/tensorflow/core/protobuf/config.proto#L431>`_.
+
+    References
+    ----------
+    *Huifeng Guo et al.* `DeepFM: A Factorization-Machine based Neural Network for CTR Prediction
+    <https://arxiv.org/pdf/1703.04247.pdf>`_.
+    """
+
     user_variables = ["linear_user_feat", "embed_user_feat"]
     item_variables = ["linear_item_feat", "embed_item_feat"]
     sparse_variables = ["linear_sparse_feat", "embed_sparse_feat"]
@@ -39,7 +86,7 @@ class DeepFM(TfBase, metaclass=ModelMeta):
     def __init__(
         self,
         task,
-        data_info=None,
+        data_info,
         loss_type="cross_entropy",
         embed_size=16,
         n_epochs=20,
@@ -51,21 +98,27 @@ class DeepFM(TfBase, metaclass=ModelMeta):
         num_neg=1,
         use_bn=True,
         dropout_rate=None,
-        hidden_units="128,64,32",
+        hidden_units=(128, 64, 32),
         multi_sparse_combiner="sqrtn",
         seed=42,
         lower_upper_bound=None,
         tf_sess_config=None,
-        with_training=True,
     ):
         super().__init__(task, data_info, lower_upper_bound, tf_sess_config)
 
         self.all_args = locals()
+        self.loss_type = loss_type
         self.embed_size = embed_size
+        self.n_epochs = n_epochs
+        self.lr = lr
+        self.lr_decay = lr_decay
+        self.epsilon = epsilon
         self.reg = reg_config(reg)
+        self.batch_size = batch_size
+        self.num_neg = num_neg
         self.use_bn = use_bn
         self.dropout_rate = dropout_config(dropout_rate)
-        self.hidden_units = list(map(int, hidden_units.split(",")))
+        self.hidden_units = hidden_units_config(hidden_units)
         self.seed = seed
         self.sparse = check_sparse_indices(data_info)
         self.dense = check_dense_values(data_info)
@@ -80,21 +133,8 @@ class DeepFM(TfBase, metaclass=ModelMeta):
             )
         if self.dense:
             self.dense_field_size = dense_field_size(data_info)
-        self._build_model()
-        if with_training:
-            self.trainer = TensorFlowTrainer(
-                self,
-                task,
-                loss_type,
-                n_epochs,
-                lr,
-                lr_decay,
-                epsilon,
-                batch_size,
-                num_neg,
-            )
 
-    def _build_model(self):
+    def build_model(self):
         tf.set_random_seed(self.seed)
         self.labels = tf.placeholder(tf.float32, shape=[None])
         self.is_training = tf.placeholder_with_default(False, shape=[])
@@ -134,25 +174,25 @@ class DeepFM(TfBase, metaclass=ModelMeta):
         linear_user_feat = tf.get_variable(
             name="linear_user_feat",
             shape=[self.n_users + 1, 1],
-            initializer=tf_truncated_normal(0.0, 0.01),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
         linear_item_feat = tf.get_variable(
             name="linear_item_feat",
             shape=[self.n_items + 1, 1],
-            initializer=tf_truncated_normal(0.0, 0.01),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
         embed_user_feat = tf.get_variable(
             name="embed_user_feat",
             shape=[self.n_users + 1, self.embed_size],
-            initializer=tf_truncated_normal(0.0, 0.01),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
         embed_item_feat = tf.get_variable(
             name="embed_item_feat",
             shape=[self.n_items + 1, self.embed_size],
-            initializer=tf_truncated_normal(0.0, 0.01),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
 
@@ -179,13 +219,13 @@ class DeepFM(TfBase, metaclass=ModelMeta):
         linear_sparse_feat = tf.get_variable(
             name="linear_sparse_feat",
             shape=[self.sparse_feature_size],
-            initializer=tf_truncated_normal(0.0, 0.01),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
         embed_sparse_feat = tf.get_variable(
             name="embed_sparse_feat",
             shape=[self.sparse_feature_size, self.embed_size],
-            initializer=tf_truncated_normal(0.0, 0.01),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
 
@@ -235,13 +275,13 @@ class DeepFM(TfBase, metaclass=ModelMeta):
         linear_dense_feat = tf.get_variable(
             name="linear_dense_feat",
             shape=[self.dense_field_size],
-            initializer=tf_truncated_normal(0.0, 0.01),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
         embed_dense_feat = tf.get_variable(
             name="embed_dense_feat",
             shape=[self.dense_field_size, self.embed_size],
-            initializer=tf_truncated_normal(0.0, 0.01),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
 
