@@ -1,13 +1,5 @@
-"""
-
-Reference: Guorui Zhou et al.  "Deep Interest Network for Click-Through Rate Prediction"
-           (https://arxiv.org/pdf/1706.06978.pdf)
-
-author: massquantity
-
-"""
+"""Implementation of DIN."""
 import numpy as np
-from tensorflow.keras.initializers import glorot_uniform
 
 from ..bases import ModelMeta, TfBase
 from ..data.sequence import get_user_last_interacted
@@ -19,11 +11,11 @@ from ..tfops import (
     tf,
     tf_dense,
 )
-from ..training import TensorFlowTrainer
+from ..torchops import hidden_units_config
 from ..utils.misc import count_params
 from ..utils.validate import (
     check_dense_values,
-    check_interaction_mode,
+    check_seq_mode,
     check_multi_sparse,
     check_sparse_indices,
     dense_field_size,
@@ -34,6 +26,71 @@ from ..utils.validate import (
 
 
 class DIN(TfBase, metaclass=ModelMeta):
+    """*Deep Interest Network* algorithm.
+
+    Parameters
+    ----------
+    task : {'rating', 'ranking'}
+        Recommendation task. See :ref:`Task`.
+    data_info : :class:`~libreco.data.DataInfo` object
+        Object that contains useful information for training and inference.
+    loss_type : {'cross_entropy', 'focal'}, default: 'cross_entropy'
+        Loss for model training.
+    embed_size: int, default: 16
+        Vector size of embeddings.
+    n_epochs: int, default: 10
+        Number of epochs for training.
+    lr : float, default 0.001
+        Learning rate for training.
+    lr_decay : bool, default: False
+        Whether to use learning rate decay.
+    epsilon : float, default: 1e-5
+        A small constant added to the denominator to improve numerical stability in
+        Adam optimizer.
+        According to the `official comment <https://github.com/tensorflow/tensorflow/blob/v1.15.0/tensorflow/python/training/adam.py#L64>`_,
+        default value of `1e-8` for `epsilon` is generally not good, so here we choose `1e-5`.
+        Users can try tuning this hyperparameter if the training is unstable.
+    reg : float or None, default: None
+        Regularization parameter, must be non-negative or None.
+    batch_size : int, default: 256
+        Batch size for training.
+    num_neg : int, default: 1
+        Number of negative samples for each positive sample, only used in `ranking` task.
+    use_bn : bool, default: True
+        Whether to use batch normalization.
+    dropout_rate : float or None, default: None
+        Probability of an element to be zeroed. If it is None, dropout is not used.
+    hidden_units : int, list of int or tuple of (int,), default: (128, 64, 32)
+        Number of layers and corresponding layer size in MLP.
+
+        .. versionchanged:: 1.0.0
+           Accept type of ``int``, ``list`` or ``tuple``, instead of ``str``.
+
+    recent_num : int or None, default: 10
+        Number of recent items to use in user behavior sequence.
+    random_num : int or None, default: None
+        Number of random sampled items to use in user behavior sequence.
+        If `recent_num` is not None, `random_num` is not considered.
+    use_tf_attention : bool, default: False
+        Whether to use TensorFlow's `attention <https://www.tensorflow.org/versions/r1.15/api_docs/python/tf/keras/layers/Attention>`_ implementation.
+        The TensorFlow attention version is simpler and faster, but doesn't follow the
+        settings in paper, whereas our implementation does.
+    multi_sparse_combiner : {'normal', 'mean', 'sum', 'sqrtn'}, default: 'sqrtn'
+        Options for combining `multi_sparse` features.
+    seed : int, default: 42
+        Random seed.
+    lower_upper_bound : tuple or None, default: None
+        Lower and upper score bound for `rating` task.
+    tf_sess_config : dict or None, default: None
+        Optional TensorFlow session config, see `ConfigProto options
+        <https://github.com/tensorflow/tensorflow/blob/v2.10.0/tensorflow/core/protobuf/config.proto#L431>`_.
+
+    References
+    ----------
+    *Guorui Zhou et al.* `Deep Interest Network for Click-Through Rate Prediction
+    <https://arxiv.org/pdf/1706.06978.pdf>`_.
+    """
+
     user_variables = ["user_feat"]
     item_variables = ["item_feat"]
     sparse_variables = ["sparse_feat"]
@@ -54,7 +111,7 @@ class DIN(TfBase, metaclass=ModelMeta):
         num_neg=1,
         use_bn=True,
         dropout_rate=None,
-        hidden_units="128,64,32",
+        hidden_units=(128, 64, 32),
         recent_num=10,
         random_num=None,
         use_tf_attention=False,
@@ -62,20 +119,25 @@ class DIN(TfBase, metaclass=ModelMeta):
         seed=42,
         lower_upper_bound=None,
         tf_sess_config=None,
-        with_training=True,
     ):
         super().__init__(task, data_info, lower_upper_bound, tf_sess_config)
 
         self.all_args = locals()
+        self.loss_type = loss_type
         self.embed_size = embed_size
+        self.n_epochs = n_epochs
+        self.lr = lr
+        self.lr_decay = lr_decay
+        self.epsilon = epsilon
         self.reg = reg_config(reg)
+        self.batch_size = batch_size
+        self.num_neg = num_neg
         self.use_bn = use_bn
         self.dropout_rate = dropout_config(dropout_rate)
-        self.hidden_units = list(map(int, hidden_units.split(",")))
+        self.hidden_units = hidden_units_config(hidden_units)
         self.use_tf_attention = use_tf_attention
-        (self.interaction_mode, self.max_seq_len) = check_interaction_mode(
-            recent_num, random_num
-        )
+        self.seq_mode, self.max_seq_len = check_seq_mode(recent_num, random_num)
+        self.recent_seqs, self.recent_seq_lens = self._set_recent_seqs()
         self.seed = seed
         self.sparse = check_sparse_indices(data_info)
         self.dense = check_dense_values(data_info)
@@ -98,26 +160,8 @@ class DIN(TfBase, metaclass=ModelMeta):
         if self.item_dense:
             # item dense col indices in all dense cols
             self.item_dense_col_indices = data_info.item_dense_col.index
-        (
-            self.user_last_interacted,
-            self.last_interacted_len,
-        ) = self._set_last_interacted()
-        self._build_model()
-        if with_training:
-            self.trainer = TensorFlowTrainer(
-                self,
-                task,
-                loss_type,
-                n_epochs,
-                lr,
-                lr_decay,
-                epsilon,
-                batch_size,
-                num_neg,
-            )
 
-    def _build_model(self):
-        self.graph_built = True
+    def build_model(self):
         tf.set_random_seed(self.seed)
         self.concat_embed, self.item_embed, self.seq_embed = [], [], []
         self._build_placeholders()
@@ -164,27 +208,27 @@ class DIN(TfBase, metaclass=ModelMeta):
         self.user_feat = tf.get_variable(
             name="user_feat",
             shape=[self.n_users + 1, self.embed_size],
-            initializer=glorot_uniform,
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
         self.item_feat = tf.get_variable(
             name="item_feat",
             shape=[self.n_items + 1, self.embed_size],
-            initializer=glorot_uniform,
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
         if self.sparse:
             self.sparse_feat = tf.get_variable(
                 name="sparse_feat",
                 shape=[self.sparse_feature_size, self.embed_size],
-                initializer=glorot_uniform,
+                initializer=tf.glorot_uniform_initializer(),
                 regularizer=self.reg,
             )
         if self.dense:
             self.dense_feat = tf.get_variable(
                 name="dense_feat",
                 shape=[self.dense_field_size, self.embed_size],
-                initializer=glorot_uniform,
+                initializer=tf.glorot_uniform_initializer(),
                 regularizer=self.reg,
             )
 
@@ -367,11 +411,11 @@ class DIN(TfBase, metaclass=ModelMeta):
             pooled_outputs = attention_scores @ keys
             return pooled_outputs
 
-    def _set_last_interacted(self):
-        user_last_interacted, last_interacted_len = get_user_last_interacted(
+    def _set_recent_seqs(self):
+        recent_seqs, recent_seq_lens = get_user_last_interacted(
             self.n_users, self.user_consumed, self.n_items, self.max_seq_len
         )
         oov = np.full(self.max_seq_len, self.n_items, dtype=np.int32)
-        user_last_interacted = np.vstack([user_last_interacted, oov])
-        last_interacted_len = np.append(last_interacted_len, [1])
-        return user_last_interacted, last_interacted_len
+        recent_seqs = np.vstack([recent_seqs, oov])
+        recent_seq_lens = np.append(recent_seq_lens, [1])
+        return recent_seqs, recent_seq_lens
