@@ -1,3 +1,4 @@
+"""Implementation of ItemCF."""
 from collections import defaultdict
 from operator import itemgetter
 
@@ -8,6 +9,35 @@ from ..bases import CfBase
 
 
 class ItemCF(CfBase):
+    """*Item Collaborative Filtering* algorithm.
+
+    Parameters
+    ----------
+    task : {'rating', 'ranking'}
+        Recommendation task. See :ref:`Task`.
+    data_info : :class:`~libreco.data.DataInfo` object
+        Object that contains useful information for training and inference.
+    sim_type : {'cosine', 'pearson', 'jaccard'}, default: 'cosine'
+        Types for computing similarities.
+    k_sim : int, default: 20
+        Number of similar items to use.
+    store_top_k : bool, default: True
+        Whether to store top k similar items after training.
+    block_size : int or None, default: None
+        Block size for computing similarity matrix. Large block size makes computation
+        faster, but may cause memory issue.
+    num_threads : int, default: 1
+        Number of threads to use.
+    min_common : int, default: 1
+        Number of minimum common items to consider when computing similarities.
+    mode : {'forward', 'invert'}, default: 'invert'
+        Whether to use forward index or invert index.
+    seed : int, default: 42
+        Random seed.
+    lower_upper_bound : tuple or None, default: None
+        Lower and upper score bound for `rating` task.
+    """
+
     def __init__(
         self,
         task,
@@ -39,6 +69,25 @@ class ItemCF(CfBase):
         self.all_args = locals()
 
     def predict(self, user, item, cold_start="popular", inner_id=False):
+        """Make prediction(s) on given user(s) and item(s).
+
+        Parameters
+        ----------
+        user : int or str or array_like
+            User id or batch of user ids.
+        item : int or str or array_like
+            Item id or batch of item ids.
+        cold_start : {'popular'}, default: 'popular'
+            Cold start strategy, ItemCF can only use 'popular' strategy.
+        inner_id : bool, default: False
+            Whether to use inner_id defined in `libreco`. For library users inner_id
+            may never be used.
+
+        Returns
+        -------
+        prediction : float or array_like
+            Predicted scores for each user-item pair.
+        """
         user_arr, item_arr = self.pre_predict_check(user, item, inner_id, cold_start)
         preds = []
         sim_matrix = self.sim_matrix
@@ -66,7 +115,6 @@ class ItemCF(CfBase):
             preds.append(pred)
         return preds[0] if len(user_arr) == 1 else preds
 
-    # all the items returned by this function will be inner_ids
     def recommend_one(self, user_id, n_rec, filter_consumed, random_rec):
         user_slice = slice(
             self.user_interaction.indptr[user_id],
@@ -117,4 +165,4 @@ class ItemCF(CfBase):
         self.topk_sim = top_k
 
     def rebuild_model(self, path, model_name, **kwargs):
-        raise NotImplementedError(f"{self.model_name} doesn't support model retraining")
+        raise NotImplementedError("`ItemCF` doesn't support model retraining")
