@@ -1,25 +1,79 @@
-"""
-
-Reference: Balazs Hidasi et al.  "Session-based Recommendations with Recurrent Neural Networks"
-           (https://arxiv.org/pdf/1511.06939.pdf)
-
-author: massquantity
-
-"""
+"""Implementation of RNN4Rec model."""
 import numpy as np
-from tensorflow.keras.initializers import glorot_normal as tf_glorot_normal
-from tensorflow.keras.initializers import truncated_normal as tf_truncated_normal
-from tensorflow.keras.initializers import zeros as tf_zeros
 
 from ..bases import EmbedBase, ModelMeta
 from ..data.sequence import get_user_last_interacted
 from ..tfops import dropout_config, reg_config, sess_config, tf, tf_dense, tf_rnn
-from ..training import RNN4RecTrainer
+from ..torchops import hidden_units_config
 from ..utils.misc import count_params
-from ..utils.validate import check_interaction_mode
+from ..utils.validate import check_seq_mode
 
 
 class RNN4Rec(EmbedBase, metaclass=ModelMeta, backend="tensorflow"):
+    """*RNN4Rec* algorithm.
+
+    .. NOTE::
+        The original paper used GRU, but in this implementation we can also use LSTM.
+
+    Parameters
+    ----------
+    task : {'rating', 'ranking'}
+        Recommendation task. See :ref:`Task`.
+    data_info : :class:`~libreco.data.DataInfo` object
+        Object that contains useful information for training and inference.
+    loss_type : {'cross_entropy', 'focal', 'bpr'}, default: 'cross_entropy'
+        Loss for model training.
+    rnn_type : {'lstm', 'gru'}, default: 'gru'
+        RNN for modeling.
+    embed_size: int, default: 16
+        Vector size of embeddings.
+    n_epochs: int, default: 10
+        Number of epochs for training.
+    lr : float, default 0.001
+        Learning rate for training.
+    lr_decay : bool, default: False
+        Whether to use learning rate decay.
+    epsilon : float, default: 1e-5
+        A small constant added to the denominator to improve numerical stability in
+        Adam optimizer.
+        According to the `official comment <https://github.com/tensorflow/tensorflow/blob/v1.15.0/tensorflow/python/training/adam.py#L64>`_,
+        default value of `1e-8` for `epsilon` is generally not good, so here we choose `1e-5`.
+        Users can try tuning this hyperparameter if the training is unstable.
+    reg : float or None, default: None
+        Regularization parameter, must be non-negative or None.
+    batch_size : int, default: 256
+        Batch size for training.
+    num_neg : int, default: 1
+        Number of negative samples for each positive sample, only used in `ranking` task.
+    dropout_rate : float or None, default: None
+        Probability of an element to be zeroed. If it is None, dropout is not used.
+    hidden_units : int, list of int or tuple of (int,), default: 16
+        Number of layers and corresponding layer size in RNN.
+
+        .. versionchanged:: 1.0.0
+           Accept type of ``int``, ``list`` or ``tuple``, instead of ``str``.
+
+    use_layer_norm : bool, default: False
+        Whether to use layer normalization.
+    recent_num : int or None, default: 10
+        Number of recent items to use in user behavior sequence.
+    random_num : int or None, default: None
+        Number of random sampled items to use in user behavior sequence.
+        If `recent_num` is not None, `random_num` is not considered.
+    seed : int, default: 42
+        Random seed.
+    lower_upper_bound : tuple or None, default: None
+        Lower and upper score bound for `rating` task.
+    tf_sess_config : dict or None, default: None
+        Optional TensorFlow session config, see `ConfigProto options
+        <https://github.com/tensorflow/tensorflow/blob/v2.10.0/tensorflow/core/protobuf/config.proto#L431>`_.
+
+    References
+    ----------
+    *Balazs Hidasi et al.* `Session-based Recommendations with Recurrent Neural Networks
+    <https://arxiv.org/pdf/1511.06939.pdf>`_.
+    """
+
     item_variables = ["item_weights", "item_biases", "input_embed"]
 
     def __init__(
@@ -27,64 +81,52 @@ class RNN4Rec(EmbedBase, metaclass=ModelMeta, backend="tensorflow"):
         task,
         data_info=None,
         loss_type="cross_entropy",
-        rnn_type="lstm",
+        rnn_type="gru",
         embed_size=16,
         n_epochs=20,
         lr=0.001,
         lr_decay=False,
         epsilon=1e-5,
-        hidden_units="16",
         reg=None,
         batch_size=256,
         num_neg=1,
         dropout_rate=None,
+        hidden_units=16,
         use_layer_norm=False,
         recent_num=10,
         random_num=None,
         seed=42,
         lower_upper_bound=None,
         tf_sess_config=None,
-        with_training=True,
     ):
         super().__init__(task, data_info, embed_size, lower_upper_bound)
 
         self.all_args = locals()
+        self.sess = sess_config(tf_sess_config)
+        self.loss_type = loss_type
         self.rnn_type = rnn_type.lower()
-        self.hidden_units = list(map(int, hidden_units.split(",")))
+        self.n_epochs = n_epochs
+        self.lr = lr
+        self.lr_decay = lr_decay
+        self.epsilon = epsilon
+        self.hidden_units = hidden_units_config(hidden_units)
         self.reg = reg_config(reg)
+        self.batch_size = batch_size
+        self.num_neg = num_neg
         self.dropout_rate = dropout_config(dropout_rate)
         self.use_ln = use_layer_norm
         self.seed = seed
-        (self.interaction_mode, self.max_seq_len) = check_interaction_mode(
-            recent_num, random_num
-        )
-        (
-            self.user_last_interacted,
-            self.last_interacted_len,
-        ) = self._set_last_interacted()
+        self.seq_mode, self.max_seq_len = check_seq_mode(recent_num, random_num)
+        self.recent_seqs, self.recent_seq_lens = self._set_recent_seqs()
         self._check_params()
-        if with_training:
-            self._build_model(loss_type)
-            self.sess = sess_config(tf_sess_config)
-            self.trainer = RNN4RecTrainer(
-                self,
-                task,
-                loss_type,
-                n_epochs,
-                lr,
-                lr_decay,
-                epsilon,
-                batch_size,
-                num_neg,
-            )
 
-    def _build_model(self, loss_type):
+    def build_model(self):
         tf.set_random_seed(self.seed)
         self.labels = tf.placeholder(tf.float32, shape=[None])
         self.is_training = tf.placeholder_with_default(False, shape=[])
         self._build_variables()
         self._build_user_embeddings()
-        if self.task == "rating" or loss_type in ("cross_entropy", "focal"):
+        if self.task == "rating" or self.loss_type in ("cross_entropy", "focal"):
             self.user_indices = tf.placeholder(tf.int32, shape=[None])
             self.item_indices = tf.placeholder(tf.int32, shape=[None])
 
@@ -95,7 +137,7 @@ class RNN4Rec(EmbedBase, metaclass=ModelMeta, backend="tensorflow"):
                 + item_bias
             )
 
-        elif loss_type == "bpr":
+        elif self.loss_type == "bpr":
             self.item_indices_pos = tf.placeholder(tf.int32, shape=[None])
             self.item_indices_neg = tf.placeholder(tf.int32, shape=[None])
             item_embed_pos = tf.nn.embedding_lookup(
@@ -126,13 +168,13 @@ class RNN4Rec(EmbedBase, metaclass=ModelMeta, backend="tensorflow"):
         self.item_biases = tf.get_variable(
             name="item_biases",
             shape=[self.n_items],
-            initializer=tf_zeros,
+            initializer=tf.zeros_initializer(),
             regularizer=self.reg,
         )
         self.item_weights = tf.get_variable(
             name="item_weights",
             shape=[self.n_items, self.embed_size],
-            initializer=tf_truncated_normal(0.0, 0.02),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
 
@@ -140,7 +182,7 @@ class RNN4Rec(EmbedBase, metaclass=ModelMeta, backend="tensorflow"):
         self.input_embed = tf.get_variable(
             name="input_embed",
             shape=[self.n_items + 1, self.hidden_units[0]],
-            initializer=tf_glorot_normal,
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
 
@@ -164,16 +206,16 @@ class RNN4Rec(EmbedBase, metaclass=ModelMeta, backend="tensorflow"):
         )
         self.user_vector = tf_dense(units=self.embed_size, activation=None)(rnn_output)
 
-    def _set_last_interacted(self):
-        user_last_interacted, last_interacted_len = get_user_last_interacted(
+    def _set_recent_seqs(self):
+        recent_seqs, recent_seq_lens = get_user_last_interacted(
             self.n_users, self.user_consumed, self.n_items, self.max_seq_len
         )
-        return user_last_interacted, last_interacted_len.astype(np.int64)
+        return recent_seqs, recent_seq_lens.astype(np.int64)
 
     def set_embeddings(self):
         feed_dict = {
-            self.user_interacted_seq: self.user_last_interacted,
-            self.user_interacted_len: self.last_interacted_len,
+            self.user_interacted_seq: self.recent_seqs,
+            self.user_interacted_len: self.recent_seq_lens,
         }
         user_vector = self.sess.run(self.user_vector, feed_dict)
         item_weights = self.sess.run(self.item_weights)
