@@ -1,15 +1,5 @@
-"""
-
-Reference: Aaron van den Oord et al. "WaveNet: A Generative Model for Raw Audio"
-           (https://arxiv.org/pdf/1609.03499.pdf)
-
-author: massquantity
-
-"""
+"""Implementation of WaveNet."""
 import numpy as np
-from tensorflow.keras.initializers import glorot_normal as tf_glorot_normal
-from tensorflow.keras.initializers import truncated_normal as tf_truncated_normal
-from tensorflow.keras.initializers import zeros as tf_zeros
 
 from ..bases import EmbedBase, ModelMeta
 from ..data.sequence import get_user_last_interacted
@@ -22,12 +12,70 @@ from ..tfops import (
     tf,
     tf_dense,
 )
-from ..training import TensorFlowTrainer
 from ..utils.misc import count_params
-from ..utils.validate import check_interaction_mode
+from ..utils.validate import check_seq_mode
 
 
 class WaveNet(EmbedBase, metaclass=ModelMeta, backend="tensorflow"):
+    """*WaveNet* algorithm.
+
+    Parameters
+    ----------
+    task : {'rating', 'ranking'}
+        Recommendation task. See :ref:`Task`.
+    data_info : :class:`~libreco.data.DataInfo` object
+        Object that contains useful information for training and inference.
+    loss_type : {'cross_entropy', 'focal'}, default: 'cross_entropy'
+        Loss for model training.
+    embed_size: int, default: 16
+        Vector size of embeddings.
+    n_epochs: int, default: 10
+        Number of epochs for training.
+    lr : float, default 0.001
+        Learning rate for training.
+    lr_decay : bool, default: False
+        Whether to use learning rate decay.
+    epsilon : float, default: 1e-5
+        A small constant added to the denominator to improve numerical stability in
+        Adam optimizer.
+        According to the `official comment <https://github.com/tensorflow/tensorflow/blob/v1.15.0/tensorflow/python/training/adam.py#L64>`_,
+        default value of `1e-8` for `epsilon` is generally not good, so here we choose `1e-5`.
+        Users can try tuning this hyperparameter if the training is unstable.
+    reg : float or None, default: None
+        Regularization parameter, must be non-negative or None.
+    batch_size : int, default: 256
+        Batch size for training.
+    num_neg : int, default: 1
+        Number of negative samples for each positive sample, only used in `ranking` task.
+    use_bn : bool, default: True
+        Whether to use batch normalization.
+    dropout_rate : float or None, default: None
+        Probability of an element to be zeroed. If it is None, dropout is not used.
+    n_filters : int, default: 16
+        Number of output filters in each CNN layer.
+    n_blocks : int, default: 1
+        Number of CNN blocks.
+    n_layers_per_block : int, default: 4
+        Number of CNN layers in each block.
+    recent_num : int or None, default: 10
+        Number of recent items to use in user behavior sequence.
+    random_num : int or None, default: None
+        Number of random sampled items to use in user behavior sequence.
+        If `recent_num` is not None, `random_num` is not considered.
+    seed : int, default: 42
+        Random seed.
+    lower_upper_bound : tuple or None, default: None
+        Lower and upper score bound for `rating` task.
+    tf_sess_config : dict or None, default: None
+        Optional TensorFlow session config, see `ConfigProto options
+        <https://github.com/tensorflow/tensorflow/blob/v2.10.0/tensorflow/core/protobuf/config.proto#L431>`_.
+
+    References
+    ----------
+    *Aaron van den Oord et al.* `WaveNet: A Generative Model for Raw Audio
+    <https://arxiv.org/pdf/1609.03499.pdf>`_.
+    """
+
     user_variables = ["user_feat"]
     item_variables = ["item_weights", "item_biases", "input_embed"]
 
@@ -54,41 +102,29 @@ class WaveNet(EmbedBase, metaclass=ModelMeta, backend="tensorflow"):
         seed=42,
         lower_upper_bound=None,
         tf_sess_config=None,
-        with_training=True,
     ):
         super().__init__(task, data_info, embed_size, lower_upper_bound)
 
         self.all_args = locals()
+        self.sess = sess_config(tf_sess_config)
+        self.loss_type = loss_type
+        self.n_epochs = n_epochs
+        self.lr = lr
+        self.lr_decay = lr_decay
+        self.epsilon = epsilon
         self.reg = reg_config(reg)
+        self.batch_size = batch_size
+        self.num_neg = num_neg
         self.dropout_rate = dropout_config(dropout_rate)
         self.use_bn = use_bn
         self.n_filters = n_filters
         self.n_blocks = n_blocks
         self.n_layers_per_block = n_layers_per_block
         self.seed = seed
-        (self.interaction_mode, self.max_seq_len) = check_interaction_mode(
-            recent_num, random_num
-        )
-        (
-            self.user_last_interacted,
-            self.last_interacted_len,
-        ) = self._set_last_interacted()
-        if with_training:
-            self._build_model()
-            self.sess = sess_config(tf_sess_config)
-            self.trainer = TensorFlowTrainer(
-                self,
-                task,
-                loss_type,
-                n_epochs,
-                lr,
-                lr_decay,
-                epsilon,
-                batch_size,
-                num_neg,
-            )
+        self.seq_mode, self.max_seq_len = check_seq_mode(recent_num, random_num)
+        self.recent_seqs, self.recent_seq_lens = self._set_recent_seqs()
 
-    def _build_model(self):
+    def build_model(self):
         tf.set_random_seed(self.seed)
         self._build_placeholders()
         self._build_variables()
@@ -116,7 +152,7 @@ class WaveNet(EmbedBase, metaclass=ModelMeta, backend="tensorflow"):
         self.user_feat = tf.get_variable(
             name="user_feat",
             shape=[self.n_users, self.embed_size],
-            initializer=tf_truncated_normal(0.0, 0.01),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
 
@@ -124,12 +160,12 @@ class WaveNet(EmbedBase, metaclass=ModelMeta, backend="tensorflow"):
         self.item_biases = tf.get_variable(
             name="item_biases",
             shape=[self.n_items],
-            initializer=tf_zeros,
+            initializer=tf.zeros_initializer(),
         )
         self.item_weights = tf.get_variable(
             name="item_weights",
             shape=[self.n_items, self.embed_size * 2],
-            initializer=tf_truncated_normal(0.0, 0.02),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
 
@@ -137,7 +173,7 @@ class WaveNet(EmbedBase, metaclass=ModelMeta, backend="tensorflow"):
         self.input_embed = tf.get_variable(
             name="input_embed",
             shape=[self.n_items + 1, self.embed_size],
-            initializer=tf_glorot_normal,
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
 
@@ -178,17 +214,17 @@ class WaveNet(EmbedBase, metaclass=ModelMeta, backend="tensorflow"):
         convs_out = tf_dense(units=self.embed_size, activation=None)(convs_out)
         self.user_vector = tf.concat([user_repr, convs_out], axis=1)
 
-    def _set_last_interacted(self):
-        user_last_interacted, last_interacted_len = get_user_last_interacted(
+    def _set_recent_seqs(self):
+        recent_seqs, recent_seq_lens = get_user_last_interacted(
             self.n_users, self.user_consumed, self.n_items, self.max_seq_len
         )
-        return user_last_interacted, last_interacted_len.astype(np.int64)
+        return recent_seqs, recent_seq_lens.astype(np.int64)
 
     def set_embeddings(self):
         feed_dict = {
             self.user_indices: np.arange(self.n_users),
-            self.user_interacted_seq: self.user_last_interacted,
-            self.user_interacted_len: self.last_interacted_len,
+            self.user_interacted_seq: self.recent_seqs,
+            self.user_interacted_len: self.recent_seq_lens,
         }
         user_vector = self.sess.run(self.user_vector, feed_dict)
         item_weights = self.sess.run(self.item_weights)
