@@ -1,13 +1,5 @@
-"""
-
-Reference: Paul Covington et al.  "Deep Neural Networks for YouTube Recommendations"
-           (https://static.googleusercontent.com/media/research.google.com/zh-CN//pubs/archive/45530.pdf)
-
-author: massquantity
-
-"""
+"""Implementation of YouTubeRanking."""
 import numpy as np
-from tensorflow.keras.initializers import truncated_normal as tf_truncated_normal
 
 from ..bases import ModelMeta, TfBase
 from ..data.sequence import get_user_last_interacted
@@ -19,11 +11,11 @@ from ..tfops import (
     tf,
     tf_dense,
 )
-from ..training import TensorFlowTrainer
+from ..torchops import hidden_units_config
 from ..utils.misc import count_params
 from ..utils.validate import (
     check_dense_values,
-    check_interaction_mode,
+    check_seq_mode,
     check_multi_sparse,
     check_sparse_indices,
     dense_field_size,
@@ -34,9 +26,72 @@ from ..utils.validate import (
 
 
 class YouTubeRanking(TfBase, metaclass=ModelMeta):
-    """
-    The model implemented mainly corresponds to the ranking phase
-    based on the original paper.
+    """*YouTubeRanking* algorithm.
+
+    .. NOTE::
+        The algorithm implemented mainly corresponds to the ranking phase
+        based on the original paper.
+
+    .. WARNING::
+        YouTubeRanking can only be used in `ranking` task.
+
+    Parameters
+    ----------
+    task : {'ranking'}
+        Recommendation task. See :ref:`Task`.
+    data_info : :class:`~libreco.data.DataInfo` object
+        Object that contains useful information for training and inference.
+    loss_type : {'cross_entropy', 'focal'}, default: 'cross_entropy'
+        Loss for model training.
+    embed_size: int, default: 16
+        Vector size of embeddings.
+    n_epochs: int, default: 10
+        Number of epochs for training.
+    lr : float, default 0.001
+        Learning rate for training.
+    lr_decay : bool, default: False
+        Whether to use learning rate decay.
+    epsilon : float, default: 1e-5
+        A small constant added to the denominator to improve numerical stability in
+        Adam optimizer.
+        According to the `official comment <https://github.com/tensorflow/tensorflow/blob/v1.15.0/tensorflow/python/training/adam.py#L64>`_,
+        default value of `1e-8` for `epsilon` is generally not good, so here we choose `1e-5`.
+        Users can try tuning this hyperparameter if the training is unstable.
+    reg : float or None, default: None
+        Regularization parameter, must be non-negative or None.
+    batch_size : int, default: 256
+        Batch size for training.
+    num_neg : int, default: 1
+        Number of negative samples for each positive sample, only used in `ranking` task.
+    use_bn : bool, default: True
+        Whether to use batch normalization.
+    dropout_rate : float or None, default: None
+        Probability of an element to be zeroed. If it is None, dropout is not used.
+    hidden_units : int, list of int or tuple of (int,), default: (128, 64, 32)
+        Number of layers and corresponding layer size in MLP.
+
+        .. versionchanged:: 1.0.0
+           Accept type of ``int``, ``list`` or ``tuple``, instead of ``str``.
+
+    recent_num : int or None, default: 10
+        Number of recent items to use in user behavior sequence.
+    random_num : int or None, default: None
+        Number of random sampled items to use in user behavior sequence.
+        If `recent_num` is not None, `random_num` is not considered.
+    multi_sparse_combiner : {'normal', 'mean', 'sum', 'sqrtn'}, default: 'sqrtn'
+        Options for combining `multi_sparse` features.
+    seed : int, default: 42
+        Random seed.
+    lower_upper_bound : tuple or None, default: None
+        Lower and upper score bound for `rating` task.
+    tf_sess_config : dict or None, default: None
+        Optional TensorFlow session config, see `ConfigProto options
+        <https://github.com/tensorflow/tensorflow/blob/v2.10.0/tensorflow/core/protobuf/config.proto#L431>`_.
+
+    References
+    ----------
+    *Paul Covington et al.* `Deep Neural Networks for YouTube Recommendations
+    <https://static.googleusercontent.com/media/research.google.com/zh-CN//pubs/archive/45530.pdf>`_.
     """
 
     user_variables = ["user_features"]
@@ -59,27 +114,32 @@ class YouTubeRanking(TfBase, metaclass=ModelMeta):
         num_neg=1,
         use_bn=True,
         dropout_rate=None,
-        hidden_units="128,64,32",
+        hidden_units=(128, 64, 32),
         recent_num=10,
         random_num=None,
         multi_sparse_combiner="sqrtn",
         seed=42,
         lower_upper_bound=None,
         tf_sess_config=None,
-        with_training=True,
     ):
         super().__init__(task, data_info, lower_upper_bound, tf_sess_config)
 
         assert task == "ranking", "YouTube models is only suitable for ranking"
         self.all_args = locals()
+        self.loss_type = loss_type
         self.embed_size = embed_size
+        self.n_epochs = n_epochs
+        self.lr = lr
+        self.lr_decay = lr_decay
+        self.epsilon = epsilon
         self.reg = reg_config(reg)
+        self.batch_size = batch_size
+        self.num_neg = num_neg
         self.use_bn = use_bn
         self.dropout_rate = dropout_config(dropout_rate)
-        self.hidden_units = list(map(int, hidden_units.split(",")))
-        self.interaction_mode, self.max_seq_len = check_interaction_mode(
-            recent_num, random_num
-        )
+        self.hidden_units = hidden_units_config(hidden_units)
+        self.seq_mode, self.max_seq_len = check_seq_mode(recent_num, random_num)
+        self.recent_seqs, self.recent_seq_lens = self._set_recent_seqs()
         self.seed = seed
         self.sparse = check_sparse_indices(data_info)
         self.dense = check_dense_values(data_info)
@@ -94,25 +154,8 @@ class YouTubeRanking(TfBase, metaclass=ModelMeta):
             )
         if self.dense:
             self.dense_field_size = dense_field_size(data_info)
-        (
-            self.user_last_interacted,
-            self.last_interacted_len,
-        ) = self._set_last_interacted()
-        self._build_model()
-        if with_training:
-            self.trainer = TensorFlowTrainer(
-                self,
-                task,
-                loss_type,
-                n_epochs,
-                lr,
-                lr_decay,
-                epsilon,
-                batch_size,
-                num_neg,
-            )
 
-    def _build_model(self):
+    def build_model(self):
         tf.set_random_seed(self.seed)
         self.user_indices = tf.placeholder(tf.int32, shape=[None])
         self.item_indices = tf.placeholder(tf.int32, shape=[None])
@@ -127,13 +170,13 @@ class YouTubeRanking(TfBase, metaclass=ModelMeta):
         user_features = tf.get_variable(
             name="user_features",
             shape=[self.n_users + 1, self.embed_size],
-            initializer=tf_truncated_normal(0.0, 0.01),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
         item_features = tf.get_variable(
             name="item_features",
             shape=[self.n_items + 1, self.embed_size],
-            initializer=tf_truncated_normal(0.0, 0.01),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
         user_embed = tf.nn.embedding_lookup(user_features, self.user_indices)
@@ -177,7 +220,7 @@ class YouTubeRanking(TfBase, metaclass=ModelMeta):
         sparse_features = tf.get_variable(
             name="sparse_features",
             shape=[self.sparse_feature_size, self.embed_size],
-            initializer=tf_truncated_normal(0.0, 0.01),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
 
@@ -213,7 +256,7 @@ class YouTubeRanking(TfBase, metaclass=ModelMeta):
         dense_features = tf.get_variable(
             name="dense_features",
             shape=[self.dense_field_size, self.embed_size],
-            initializer=tf_truncated_normal(0.0, 0.01),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
 
@@ -227,11 +270,11 @@ class YouTubeRanking(TfBase, metaclass=ModelMeta):
         )
         self.concat_embed.append(dense_embed)
 
-    def _set_last_interacted(self):
-        user_last_interacted, last_interacted_len = get_user_last_interacted(
+    def _set_recent_seqs(self):
+        recent_seqs, recent_seq_lens = get_user_last_interacted(
             self.n_users, self.user_consumed, self.n_items, self.max_seq_len
         )
         oov = np.full(self.max_seq_len, self.n_items, dtype=np.int32)
-        user_last_interacted = np.vstack([user_last_interacted, oov])
-        last_interacted_len = np.append(last_interacted_len, [1])
-        return user_last_interacted, last_interacted_len
+        recent_seqs = np.vstack([recent_seqs, oov])
+        recent_seq_lens = np.append(recent_seq_lens, [1])
+        return recent_seqs, recent_seq_lens
