@@ -1,4 +1,5 @@
 from .base import Base
+from .cf_base import CfBase
 from .embed_base import EmbedBase
 from .gensim_base import GensimBase
 from .meta import ModelMeta
@@ -6,6 +7,7 @@ from .tf_base import TfBase
 
 __all__ = [
     "Base",
+    "CfBase",
     "EmbedBase",
     "GensimBase",
     "ModelMeta",
