@@ -1,17 +1,17 @@
-from .batch_unit import PointwiseBatch, PairwiseBatch
+from .batch_unit import PairwiseBatch, PointwiseBatch
 from .data_sampler import (
     DataGenerator,
     PairwiseDataGenerator,
-    PointwiseDataGenerator,
     PairwiseRandomWalkGenerator,
+    PointwiseDataGenerator,
 )
 from .negatives import (
+    neg_probs_from_frequency,
     negatives_from_out_batch,
     negatives_from_popular,
     negatives_from_random,
     negatives_from_unconsumed,
-    neg_probs_from_frequency,
-    pos_probs_from_frequency
+    pos_probs_from_frequency,
 )
 from .random_walks import bipartite_neighbors, bipartite_neighbors_with_weights
 
