@@ -50,16 +50,14 @@ def rebuild_torch_model(self, path, model_name):
 
     for name, param in self.torch_model.named_parameters():
         if name in user_params:
-            param.index_copy_(
-                0, torch.arange(self.data_info.old_n_users), user_params[name]
-            )
+            index = torch.arange(self.data_info.old_n_users, device=self.device)
+            param.index_copy_(0, index, user_params[name])
         elif name in item_params:
-            param.index_copy_(
-                0, torch.arange(self.data_info.old_n_items), item_params[name]
-            )
+            index = torch.arange(self.data_info.old_n_items, device=self.device)
+            param.index_copy_(0, index, item_params[name])
         elif name in sparse_params:
             old_indices, old_values = get_sparse_indices_values(
-                self.data_info, sparse_params[name]
+                self.data_info, sparse_params[name], self.device
             )
             param.index_copy_(0, old_indices, old_values)
 
@@ -70,6 +68,7 @@ def rebuild_torch_model(self, path, model_name):
             self.data_info.old_n_users,
             self.n_users,
             self.embed_size,
+            self.device,
         )
     for i in item_param_indices:
         optimizer_item_state = optimizer_state_dict["state"][i]
@@ -78,17 +77,18 @@ def rebuild_torch_model(self, path, model_name):
             self.data_info.old_n_items,
             self.n_items,
             self.embed_size,
+            self.device,
         )
     for i in sparse_param_indices:
         optimizer_sparse_state = optimizer_state_dict["state"][i]
         assign_adam_sparse_states(
-            optimizer_sparse_state, self.data_info, self.embed_size
+            optimizer_sparse_state, self.data_info, self.embed_size, self.device
         )
 
     self.trainer.optimizer.load_state_dict(optimizer_state_dict)
 
 
-def get_sparse_indices_values(data_info, sparse_param_tensor):
+def get_sparse_indices_values(data_info, sparse_param_tensor, device):
     indices = list(range(len(sparse_param_tensor)))
     # remove oov indices
     for i in data_info.old_sparse_oov:
@@ -99,26 +99,35 @@ def get_sparse_indices_values(data_info, sparse_param_tensor):
     for offset, size in zip(data_info.sparse_offset, data_info.old_sparse_len):
         if size != -1:
             indices.extend(range(offset, offset + size))
-    indices = torch.tensor(indices, dtype=torch.long)
+    indices = torch.tensor(indices, dtype=torch.long, device=device)
     return indices, sparse_param_tensor
 
 
-def assign_adam_optimizer_states(state, old_num, new_num, embed_size):
-    new_first_moment = nn.init.zeros_(torch.empty(new_num, embed_size))
-    new_first_moment.index_copy_(0, torch.arange(old_num), state["exp_avg"])
+def assign_adam_optimizer_states(state, old_num, new_num, embed_size, device):
+    index = torch.arange(old_num, device=device)
+    new_first_moment = nn.init.zeros_(torch.empty(new_num, embed_size, device=device))
+    new_first_moment.index_copy_(0, index, state["exp_avg"])
     state["exp_avg"] = new_first_moment
-    new_second_moment = nn.init.zeros_(torch.empty(new_num, embed_size))
-    new_second_moment.index_copy_(0, torch.arange(old_num), state["exp_avg_sq"])
+    new_second_moment = nn.init.zeros_(torch.empty(new_num, embed_size, device=device))
+    new_second_moment.index_copy_(0, index, state["exp_avg_sq"])
     state["exp_avg_sq"] = new_second_moment
 
 
-def assign_adam_sparse_states(state, data_info, embed_size):
+def assign_adam_sparse_states(state, data_info, embed_size, device):
     sparse_size = sparse_feat_size(data_info)
-    new_first_moment = nn.init.zeros_(torch.empty(sparse_size, embed_size))
-    old_indices, old_values = get_sparse_indices_values(data_info, state["exp_avg"])
+    new_first_moment = nn.init.zeros_(
+        torch.empty(sparse_size, embed_size, device=device)
+    )
+    old_indices, old_values = get_sparse_indices_values(
+        data_info, state["exp_avg"], device
+    )
     new_first_moment.index_copy_(0, old_indices, old_values)
     state["exp_avg"] = new_first_moment
-    new_second_moment = nn.init.zeros_(torch.empty(sparse_size, embed_size))
-    old_indices, old_values = get_sparse_indices_values(data_info, state["exp_avg_sq"])
+    new_second_moment = nn.init.zeros_(
+        torch.empty(sparse_size, embed_size, device=device)
+    )
+    old_indices, old_values = get_sparse_indices_values(
+        data_info, state["exp_avg_sq"], device
+    )
     new_second_moment.index_copy_(0, old_indices, old_values)
     state["exp_avg_sq"] = new_second_moment
